/**
 * Copyright (C) (2010-2018) Vadim Biktashev, Irina Biktasheva et al. 
 * (see ../AUTHORS for the full list of contributors)
 *
 * This file is part of Beatbox.
 *
 * Beatbox is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Beatbox is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beatbox.  If not, see <http://www.gnu.org/licenses/>.
 */

/* Wrap around ezview utility version 1.7 based on Mantel and Barkley's EZScroll visualizer */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h> 
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <GL/gl.h>
#include <GL/glx.h>

#include "system.h"
#include "beatbox.h"

#include "device.h"

#include "state.h"
#include "qpp.h"
#include "bikt.h"
#include "k_.h"

#define OWN
#include "ezview.h" /* STR is defined in there! */


RUN_HEAD(ezview)
{
  int rc;
  
  glXMakeCurrent(S->theDisplay, S->theWindow, S->theGLXContext);
  Make_lists(S);
  Draw(S);
  rc=Event_check(S);
  if (S->write_images) Save_image(S);

  return(rc==0);
} RUN_TAIL(ezview);
/* ========================================================================= */

DESTROY_HEAD(ezview)
{
  QuitX(S);
  if (S->filament) fclose(S->filament);
} DESTROY_TAIL(ezview);
/* ========================================================================= */

CREATE_HEAD(ezview)
{
  /* Initialize quasi-global computable variables that need it */
  #define _(Type,Name,Init) S->Name=Init;
  #include "ezini.h"
  #undef _

  /* Accept the user-defined parameters */
  #define nonereal RNONE
  #define noneint INONE
  #define acceptreal(b,c,d,e) if (!acceptr(#b"=",&(S->b),c,d,e,w)) return(0); real b=S->b
  #define acceptint(b,c,d,e) if (!accepti(#b"=",&(S->b),c,d,e,w)) return(0); int b=S->b 
  #define _(Type,Name,Dflt,miN,maX) \
  accept##Type(Name,Dflt,miN,maX);
  #include "ezfix.h"
  #undef _
  #define _(Type,Name,Init,reRead,reView,reLight,reMake,reDraw)	\
  accept##Type(Name,Init,none##Type,none##Type);
  #include "ezpar.h"
  #undef _
  #undef acceptreal
  #undef acceptint
  #undef noneint
  #undef nonereal

  /* Accept other paramerters */
  ACCEPTF(filament,"wt","");
  ACCEPTS(images,"");
  if (*images) {
    ACCEPTS(imagescode,"t");
    k_on();
    S->imagescompiled=compile(imagescode,deftb,t_real); CHK(imagescode);
    k_off();
  }

  /* Some checks */
  switch (color_mode) {
  case 0: case 1: case 2: case 3:
    ASSERT(0<=rlayer); ASSERT(rlayer<vmax); ASSERT(rmax!=rmin);
    break;
  case 4: case 5:
    ASSERT(0<=rlayer); ASSERT(rlayer<vmax); ASSERT(rmax!=rmin);
    ASSERT(0<=glayer); ASSERT(glayer<vmax); ASSERT(gmax!=gmin);
    ASSERT(0<=blayer); ASSERT(blayer<vmax); ASSERT(bmax!=bmin);
    ASSERT(0<=alayer); ASSERT(alayer<vmax); ASSERT(amax!=amin);
    break;
  }
  if (show_surface!=0 && amax<=0)
    MESSAGE("/* WARNING: show_surface=%d, alphamax=%lg, the surface will not be visible */\n"
	    ,(int)show_surface,(double)alphamax);
  if ( (show_filament!=0 || write_filament!=0) && layer1==layer2)
    MESSAGE("/* WARNING: show_filament=%d, write_filament=%d, but layer1=%d = layer2=%d, the filament will not be found */\n",
	    (int)show_filament,(int)write_filament,(int)layer1,(int)layer2);
  if (write_images && (0==(*images)))
    MESSAGE("/* WARNING: write_images=%d but not images template specified; no images will be written */\n",
	    (int)write_images);

  if NOT(Draw_ini(S)) return FAILURE;
} CREATE_TAIL(ezview,0);
/* ========================================================================= */

